@@ -26,8 +26,6 @@ namespace types
   template <class T>
   using container = std::vector<T, utils::allocator<T>>;
 
-  static const size_t DEFAULT_LIST_CAPACITY = 16;
-
   /* forward declaration */
   struct empty_list;
   template <class T>
@@ -38,26 +36,6 @@ namespace types
   struct ndarray;
   template <class... Tys>
   struct pshape;
-  template <class T>
-  struct is_list {
-    static const bool value = false;
-  };
-  template <class T>
-  struct is_list<list<T>> {
-    static const bool value = true;
-  };
-  template <class T, class S>
-  struct is_list<sliced_list<T, S>> {
-    static const bool value = true;
-  };
-  template <class T, size_t N>
-  struct is_list<static_list<T, N>> {
-    static const bool value = true;
-  };
-
-  /* for type disambiguification */
-  struct single_value {
-  };
 
   /* list view */
   template <class T, class S = slice>
@@ -104,10 +82,10 @@ namespace types
     static const bool is_flat = std::is_same<slice, S>::value;
     static const bool is_strided = std::is_same<slice, S>::value;
 
-    using shape_t = types::array<long, value>;
+    using shape_t = types::array_tuple<long, value>;
     template <size_t I>
-    auto shape() const
-        -> decltype(details::extract_shape(*this, utils::int_<I>{}))
+    auto shape() const -> decltype(details::extract_shape(*this,
+                                                          utils::int_<I>{}))
     {
       return details::extract_shape(*this, utils::int_<I>{});
     }
@@ -193,6 +171,7 @@ namespace types
   template <class T>
   class list
   {
+    static constexpr size_t DEFAULT_CAPACITY = 16;
 
     // data holder
     typedef
@@ -236,7 +215,6 @@ namespace types
     list(InputIterator start, InputIterator stop);
     list(empty_list const &);
     list(size_type sz);
-    list(T const &value, single_value, size_type sz = 1);
     list(std::initializer_list<T> l);
     list(list<T> &&other);
     list(list<T> const &other);
@@ -391,7 +369,7 @@ namespace types
     intptr_t id() const;
 
     long count(T const &x) const;
-    using shape_t = array<long, value>;
+    using shape_t = array_tuple<long, value>;
     template <size_t I>
     long shape() const
     {
@@ -439,7 +417,7 @@ namespace types
     static const size_t value = 1;
     static const bool is_vectorizable = false;
     static const bool is_strided = false;
-    using shape_t = types::array<long, value>;
+    using shape_t = types::array_tuple<long, value>;
     typedef char value_type;
 
     typedef empty_iterator iterator;
